-- xenjine layer 2  : tutorial sql 2
-- c2007 : lukas irwin : sysop [ at ] xhrit.com
-- editwindow.lua

editwindow = { ID = 0 }

print "[editwindow]"

-- creation
function editwindow:new (o)
	-- print "[pawn] new pawn"
	o = o or {}
	setmetatable(o, self)
	self.__index = self

	self.window_elements = {}
	self.caption = "" -- TODO: figure out how to set this?

	self.window_elements[1] = sim:addWindow( 100, xcore.x2drect(10, 10, 300, 300), self.caption )

	self.ec = 0 -- dynamic element count
	self.rows = 0 -- total number ov elements
	self.popup_size = 360

	self.edit_element = -1

	return o
end

function editwindow:destroy ()
	print "[editwindow] destroy"
	i = table.getn( self.window_elements )
	for n = i, 1, -1  do
		print("destroy element #" .. n)
		self.window_elements[ n ]:destroy()
		table.remove ( self.window_elements, n )
	end
end

function editwindow:addMessage ( message ) -- adds a message
	-- print "editwindow:addConfirmMessage"
	self.rows = self.rows + 2

	i = table.getn( self.window_elements ) + 1
	self.window_elements[ i ] = sim:addText( 101, message , 20, 30, self.popup_size-10, 200 )
	self.window_elements[ i ]:setParent( self.window_elements[1] )
	self.window_elements[ i ]:setColour( xcore.xcolour(255, 0, 0, 0) )
	-- print "/editwindow:addConfirmMessage"
end

function editwindow:addEditBox () -- adds a text box
	print "editwindow:addEditBox"
	self.rows = self.rows + 4

	i = table.getn( self.window_elements ) + 1
	self.edit_element = i
	self.window_elements[ i ] = sim:addEditbox( 102, xcore.x2drect( 20, 30, 240, 120 ) )
	self.window_elements[ i ]:setParent( self.window_elements[1] )
	self.window_elements[ i ]:setMultiLine( true )
	-- self.window_elements[ i ]:setColour( xcore.xcolour(255, 0, 0, 0) )
	print "/editwindow:addConfirmMessage"
end

function editwindow:getEditText() -- gets text from last textbox added
	print "editwindow:getEditText"
	if self.edit_element ~= -1 then
		a = self.window_elements[ self.edit_element ]:getText()
		self.edit_element = -1
		return a
	else
		print ( "error, no edit box in window" )
	end
	return nil
end

function editwindow:addInputElement ( key, value ) -- adds an input with key and value pair
	-- print "editwindow:addInputElement"
	self.rows = self.rows + 1

	row_height = 26
	x = 20
	y = self.rows * row_height

	i = table.getn( self.window_elements ) + 1
	-- add label
	self.window_elements[ i ] = sim:addText( 101, key ,  x, y, (x+(self.popup_size/2))-10, y+row_height  )
	self.window_elements[ i ]:setParent( self.window_elements[1] )
	self.window_elements[ i ]:setColour( black )
	x = self.popup_size/2

	-- add edit box
	i = table.getn( self.window_elements ) + 1
	-- print ("i:" .. i)
	self.window_elements[ i ] = sim:addEditbox( 420,  xcore.x2drect(  x, y, (x+(self.popup_size/2))-10, y+row_height ) )
	self.window_elements[ i ]:setParent( self.window_elements[1] )
	self.window_elements[ i ]:setText( value )
end

function editwindow:addButtons () -- adds accept / cancel buttons
	-- print "editwindow:addButtons"
	self.rows = self.rows + 1

	row_height = 26
	x = 20
	y = self.rows * row_height
	i = table.getn( self.window_elements ) + 1
	self.window_elements[ i ] = sim:addButton( 103, xcore.x2drect(  x, y, (x+(self.popup_size/2))-10, y+row_height  ), "Accept" )
	self.window_elements[ i ]:setParent( self.window_elements[1] )

	x = self.popup_size/2
	i = table.getn( self.window_elements ) + 1
	self.window_elements[ i ] = sim:addButton( 102, xcore.x2drect(  x, y, (x+(self.popup_size/2))-10, y+row_height  ), "Cancel" )
	self.window_elements[ i ]:setParent( self.window_elements[1] )

	sim:bindGUI ( "window_cancel", 102, xcore.XGUI_BUTTON_CLICKED )
	sim:bindGUI ( "window_accept", 103, xcore.XGUI_BUTTON_CLICKED )

	self.window_elements[ 1 ]:setSize( self.popup_size, y+30 )
end

function editwindow:getValue ( key ) -- gets value from input via key
	-- print ( "getValue " .. key )
	i = table.getn( self.window_elements ) - 1
	-- print ("no elements" .. i)
	for n = 1, i  do
		-- print ("n:" .. n )
		if self.window_elements[n]:getType() == xcore.XELEMENT_TYPE_TEXT then
			-- print "type match"
			if self.window_elements[ n ]:getText() == key then
				-- print ("element found, return :" .. self.window_elements[ n + 1 ]:getText())
				return self.window_elements[ n + 1 ]:getText()
			end
		end
	end
	-- print "value not found"
end

-- get dynamic data; we will take advantage ov one ov lua' nifty quirks
-- and return TWO! arrays, one with keys and one with datatypes.
function editwindow:getDynamicData()
	-- print "getDynamicData"
	self.keys = {}
	self.types = {}
	i = table.getn( self.window_elements ) - 1
	-- print ("no elements" .. i)
	for n = 1, i  do
		-- print ("n:" .. n )
		if self.window_elements[n]:getType() == xcore.XELEMENT_TYPE_COMBOBOX then
			p = table.getn( self.keys ) + 1
			self.keys[ p ] = self.window_elements[ n -1 ]:getText()
			self.types[ p ] =  self.window_elements[ n ]:getItem( self.window_elements[ n ]:getSelected() )
		end
	end
	-- print "returning dynamic data"
	return self.keys, self.types
end

-- this will count the number ov dynamic fields
function editwindow:getDynamicElementCount()
	-- print "getDynamicElementCount"
	-- print ("ec:" .. self.ec)
	return self.ec
end

function editwindow:addDynamicElement()
	-- print "addDynamicElement"
 	self.ec =  self.ec + 1

	row_height = 26
	self.rows = self.rows + 1
	i = table.getn( self.window_elements ) + 1
	x = 20
	y = self.rows * row_height

	-- editbox for user to input key
	self.window_elements[ i ] = sim:addEditbox( 600 + i,  xcore.x2drect( x, y, (x+(self.popup_size/2)), y+row_height ) )
	self.window_elements[ i ]:setParent( self.window_elements[1] )

	i = table.getn( self.window_elements ) + 1
	x = (self.popup_size/2)
	-- combobox for user to select value's datatype
	self.window_elements[ i ] = sim:addCombobox( 600 + i, xcore.x2drect( x, y, (x+(self.popup_size/2))-10, y+row_height ) )
	self.window_elements[ i ]:setParent( self.window_elements[1] )
	self.window_elements[ i ]:addItem( "Integer" )
	self.window_elements[ i ]:addItem( "Float" )
	self.window_elements[ i ]:addItem( "Text" )
	self.window_elements[ i ]:addItem( "Primary Key" )

	-- add buttons to remove field?
end

function editwindow:removeDynamicElement()
	-- print "removeDynamicElement"
	-- TODO: make this work
end

function editwindow:removeButtons()
	print "removeButton"
	i = table.getn( self.window_elements )
	print ( "no elements" .. i )
	for n = 1, i  do
		print ("n:" .. n )
		if self.window_elements[n]:getType() == xcore.XELEMENT_TYPE_BUTTON then
			print ("type match")
			self.window_elements[ n ]:destroy()
			table.remove ( self.window_elements, n )
			return 0
		end
	end
	return 1
end

function editwindow:addDynamicElementButton () -- adds a button to add a dynamic element
	-- print "addDynamicElementButton"
	self.rows = self.rows + 1

	row_height = 26
	x = 20
	y = self.rows * row_height
	i = table.getn( self.window_elements ) + 1
	self.window_elements[ i ] = sim:addButton( 104, xcore.x2drect(  x, y, (self.popup_size/2), y+row_height  ), "Add Element" )
	self.window_elements[ i ]:setParent( self.window_elements[1] )

	sim:bindGUI ( "window_add_element", 104, xcore.XGUI_BUTTON_CLICKED )
end

function editwindow:setSize ( w, h)
	self.window_elements[1]:setSize( w, h )
end

function editwindow:setText ( t )
	self.window_elements[1]:setText( t )
end