-- Dungeon Tactics
-- Copyright 2012 lukas w. irwin [ lukas.w.irwin@gmail.com ]
-- This program is free software under the terms of the GNU General Public License version 3 or later.
-- tile.lua

tile = { ID = 0 }

-- tile types
TILE_TYPE_CLEAR        = 0
TILE_TYPE_ROUGH        = 1
TILE_TYPE_START        = 2
TILE_TYPE_EXIT         = 3
TILE_TYPE_BLOCK        = 4
TILE_TYPE_UNUSED       = 5
TILE_TYPE_ROAD         = 6
TILE_TYPE_ROAD_EDGE    = 7
TILE_TYPE_ROAD_OUTTER  = 8  -- CORNER
TILE_TYPE_ROAD_INNER   = 9  -- CORNER
TILE_TYPE_IMPASS       = 10

TILESET_DUNGEON  = 0
TILESET_FOREST   = 1
TILESET_CAVE     = 2
TILESET_MAZE     = 3
TILESET_CASTLE   = 4

-- creation
function tile:new (o)
	-- print "[tile] new"
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	self.tile_id       = 0
	self.tile_type     = TILE_TYPE_UNUSED

	o.position = xcore.x3dpoint( 0, 0, 0 )
	o.rotation = xcore.x3dpoint( 0, 0, 0 )
	o.grid = xcore.x2dpoint( 0, 0 )

	self.valid = 0
	self.text = ""
	return o
end

function tile:setType( p )
	-- print "[tile] setType"
	self.tile_type = p
	if self.mesh ~= nil then
		self.mesh:destroy()
		self.mesh = nil
		self.selector:destroy()
	end
	if self.billboard ~= nil then
		self.billboard:destroy()
	end
	
	self.tileset_tile = game_map:getTile( p )
	if self.tileset_tile == -1 then print("[tile] Undefined tile in tileset") return nil end
	-- print("[tile] tile name : " .. game_map.tileset[ self.tileset_tile ]["name"] )	
	self.mesh = sim:addMesh ( self.tile_id,  path .. "resources/models/" .. game_map.tileset[ self.tileset_tile ]["model"] )
	self.mesh:setMaterialFlag( xcore.XMF_LIGHTING, false )
	self.mesh:setMaterialFlag( xcore.XMF_ANISOTROPIC_FILTER, true )
	-- self.mesh:setScale( xcore.x3dpoint( 5, 5*game_map.tileset[ self.tileset_tile ]["scale"], 5 ) )
	self.mesh:setScale( xcore.x3dpoint( 5, 5, 5 ) )
	self.mesh:addTexture( path .. "resources/textures/" .. game_map.tileset[ self.tileset_tile ]["texture"], 0 )		

	-- print("tile bb : " .. game_map.tileset[ self.tileset_tile ]["billboard"] )
	
	if game_map.tileset[ self.tileset_tile ]["billboard"] ~= "0" then
		self:setBillboard( game_map.tileset[self.tileset_tile ]["billboard"] )
	end
	
	self.tile_height = game_map.tileset[ self.tileset_tile ]["height"]
	
	self.selector = sim:addCubeMesh ( self.tile_id )
	self.selector:setMaterialFlag( xcore.XMF_LIGHTING, false )
	self.selector:setScale( xcore.x3dpoint( 1, 0, 1 ) )
	self.selector:addTexture( path .. "resources/interface/green.png", 0 )
	self.selector:setVisible(false)

	if self.label == nil then self:addText() end
	self:setPosition( self.position )
end

function tile:setPosition( p )
	-- print "[tile] setPosition"
	self.position = p
	if self.label ~= nil then	
		self.label:setPosition( xcore.x3dpoint( self.position.X, self.position.Y+1, self.position.Z ) )
	end

	if self.mesh ~= nil then
		self.mesh:setPosition( xcore.x3dpoint( p.X, p.Y, p.Z ) )
		self.selector:setPosition( xcore.x3dpoint( p.X, p.Y+0.1, p.Z ) )
	end
	
	if self.billboard ~= nil then
		self.billboard:setPosition( xcore.x3dpoint( p.X, p.Y+8, p.Z ) )
	end
end

function tile:getPosition () return self.position end

function tile:setRotation ( n )
	-- print ( "[tile] setRotation : " .. n )
	if self.mesh ~= nil then
		if n == 1 then
			self.mesh:setRotation( xcore.x3dpoint( 0, 0, 0 ) )
		elseif n == 2 then
			-- print("2")
			self.mesh:setRotation( xcore.x3dpoint( 0, 180, 0 ) )
		elseif n == 3 then
			self.mesh:setRotation( xcore.x3dpoint( 0, 90, 0 ) )
		elseif n == 4 then
			self.mesh:setRotation( xcore.x3dpoint( 0, 270, 0 ) )
		end
	end
end

function tile:setGraphic ( n )
	if self.mesh ~= nil then
		self.mesh:addTexture( path .. "resources/textures/" ..  n, 0 )
	end
end

function tile:setBillboard( n )
	-- print "[tile] setBillboard"
	-- print("n: " .. n)
	if self.billboard == nil then
		self.billboard = sim:addBillboard( -1, xcore.x2dpoint( 16, 16 ) )
		self.billboard:addTexture( path .. "resources/textures/" ..  n, 0 )
		self.billboard:setMaterialFlag( xcore.XMF_LIGHTING, false )
		self.billboard:setMaterialFlag( xcore.XMF_ANISOTROPIC_FILTER, true )
		self.billboard:setMaterialType( xcore.XMT_TRANSPARENT_ALPHA_CHANNEL )
		self.billboard:setPosition( self.position )
	end
end

function tile:setGrid( p ) self.grid = p end
function tile:getGrid () return	self.grid end
function tile:setID( p ) self.tile_id = p end
function tile:getID () return self.tile_id end

function tile:destroy ()
	-- print "[tile] destroy"
	self.label:setText( "" )
	self.label:destroy()
	if self.mesh ~= nil then
		self.mesh:destroy()
		self.selector:destroy()
	end
	if self.billboard ~= nil then
		self.billboard:destroy()
	end	
end

function tile:addText()
	font = sim:getBuiltInFont()
	self.label = sim:addTextNode( self.ID, font, "", xcore.xcolour( 255, 0, 0, 0 ) )
	self.label:setPosition( xcore.x3dpoint( self.position.X, self.position.Y+1, self.position.Z ) )
end

function tile:setText( t )
	-- print ( "[tile_".. self.ID .."] set Text : " .. t )
	self.text = t
end

-- end of file