-- xenjine layer 2 physics tutorial 1
-- c2007 lukas irwin sysop [ at ]xhrit.com
-- tutorial_physics_1.module

-- required
function init()
	-- start a simulation
	-- require "module.settings" for video options
	sim = xcore.xsimulation ( xcore.XDT_OPENGL , 800, 600, 32, false, true, true )
	sim:setWindowCaption( "Tutorial physics 1 :  Box Drop" )

	-- add some static text
	text = [[
	Xenos Engine Physics Demo 1 : Box Drop
	
	arrow keys to move
	mouse to look
	space to drop a box
	r to reset boxes		
	w to toggle wireframe
	esc to quit
	]]

	sim:addText( 0, text, 50, 50, 600, 600)

	-- add a fps style camera
	cam = sim:addCamera ( 0, xcore.CAMERA_FPS, 0,30,-40, 0,5,0)
	np = xcore.x3dpoint( 2500, 400, 2500  )
	cam:setPosition( np )
	cam:setFarValue( 64000 )

	-- start the dynamics
	dyn = xcore.xdynamics()
	dyn:setAutoDisable( false )
	dyn:setGravity( xcore.x3dpoint( 0, -10, 0 ) )
	dyn:setERP ( 0.8 )
	dyn:setCFM ( 0.001 )
	dyn:setTimeStep( 0.05 )
	dyn:setMode ( xcore.XDYNAMICS_FAST )

	print "add terrain"
	-- add terrain
	terrain = sim:addTerrainMesh( 10, "modules/tutorial_physics_1/resources/lake_height.bmp" )
	terrain:addTexture( "modules/tutorial_physics_1/resources/lake_texture.bmp", 0 )
	terrain:setScale( xcore.x3dpoint( 25, 5, 25 ) )
	terrain:setMaterialFlag( xcore.XMF_LIGHTING, false )

	-- create geometry for terrain
	body_terrain = xcore.xbody() -- sim:addBody ( 0, mesh_file, tex_file )??
	dyn:createBody( body_terrain )
	dyn:createTrimeshGeomFromTerrain( body_terrain, terrain, xcore.x3dpoint( 0, 0, 0 ), xcore.x3dpoint( 0, 0, 0 ), false )

	print "add sky"
	-- add a skybox (top, bottom, left, right, front, back)
	sky = sim:addSkybox(500, "modules/tutorial_physics_1/resources/skyrender0003.bmp",
			"modules/tutorial_physics_1/resources/skyrender0006.bmp",
			"modules/tutorial_physics_1/resources/skyrender0005.bmp",
			"modules/tutorial_physics_1/resources/skyrender0002.bmp",
			"modules/tutorial_physics_1/resources/skyrender0001.bmp",
			"modules/tutorial_physics_1/resources/skyrender0004.bmp")

	print "bind keys"
	-- bind keys
	sim:bindkey ( "makebox",  xcore.XKEY_SPACE  )     -- make a box!
	sim:bindkey ( "quit",  xcore.XKEY_ESCAPE  )       -- exit demo
	sim:bindkey ( "wireframe",  xcore.XKEY_W  )       -- toggle wireframes
	sim:bindkey ( "reset",  xcore.XKEY_R  )           -- reset simulation

	wireframe = 0
	time = 0
	boxes = {}
	bodies = {}
end

function render()
	-- render simulation tree
	sim:render()
end

function update()
	-- print "update"
	sim:update()
	dyn:update()

	time = time + 1

	if sim:isKey( "quit" ) then sim:destroy() end
	if sim:isKeyUp( "wireframe" ) then toggle_wireframe() end
	if sim:isKeyUp( "reset" ) then reset() end
	if sim:isKeyUp( "makebox" ) then makebox() end

	-- synch physics to graphics
	i = table.getn( boxes )
	for n = 1, i do
		boxes[n]:setPosition( bodies[n]:getPosition() ) -- should be updated automaticly with a link
		boxes[n]:setRotation( bodies[n]:getRotation() )
	end

end

function toggle_wireframe()
	-- print "[tut_p1] toggle_wireframe"
	if wireframe == 0 then
		terrain:setMaterialFlag( xcore.XMF_WIREFRAME, true )
		wireframe = 1
	else
		terrain:setMaterialFlag( xcore.XMF_WIREFRAME, false )
		wireframe = 0
	end
end

function reset()
	local object_count = table.getn( boxes )
	for i=object_count,1,-1 do
		boxes[i]:destroy()
		table.remove ( boxes, i )

		dyn:destroyGeom( bodies[i] )
		dyn:destroyBody( bodies[i] )

		table.remove ( bodies, i )		
	end
end

function makebox()
	if time > 300 then
		time = 0;
		-- print "make box"
		-- create a 3d box
		i = table.getn( boxes ) + 1
		tex_file = "modules/tutorial_physics_1/resources/rockwall.bmp"
		boxes[ i ] = sim:addCubeMesh ( 0 )
		boxes[ i ]:addTexture( tex_file, 0 )
		np = cam:getPosition()
		boxes[ i ]:setPosition( np )
		boxes[ i ]:setScale( xcore.x3dpoint( 1, 1, 1 ) )
		boxes[ i ]:setMaterialFlag( xcore.XMF_LIGHTING, false )
	
		-- create geometry for box
		bodies[i] = xcore.xbody() -- sim:addBody ( 0, mesh_file, tex_file )??
		dyn:createBody( bodies[i] )
		dyn:createBoxGeom( bodies[i], xcore.x3dpoint( 10, 10, 10 ), np, xcore.x3dpoint( 0, 0, 0 ) ) -- size, pos, rot
		bodies[i]:setMU( 100 )
	end
end

-- end of file