-- xenjine layer 2 physics tutorial 2
-- c2007-2009 lukas irwin : sysop [ at ]xhrit.com
-- tutorial_physics_2.module

-- required
function init()
	-- start a simulation
	-- sim = xcore.xsimulation ( xcore.XDT_OPENGL )
	sim = xcore.xsimulation ( xcore.XDT_OPENGL , 800, 600, 16, false, true, true )
	sim:setWindowCaption( "Tutorial physics 2 :  2d" )

	-- add some static text
	text = [[
	Xenos engine physics demo 2 : 2d
	
	arrow keys move
	spacebar to shoot
	+ and - to zoom in and out
	esc to exit
	]]
	sim:addText( 0, text, 50, 50, 600, 600)

	-- add a static camera
	zoom = 200;
	cam = sim:addCamera ( 0, xcore.CAMERA_STATIC, 0, 0, zoom, 0,0,0)

	-- start the dynamics
	dyn = xcore.xdynamics()
	dyn:setAutoDisable( false )
	dyn:setERP ( 0.8 )
	dyn:setCFM ( 0.001 )
	dyn:setTimeStep( 0.01 )
	dyn:setMode ( xcore.XDYNAMICS_FAST )

	tex_file = "modules/tutorial_physics_2/resources/y23.bmp"
	box = sim:addCubeMesh ( 0 )
	box:addTexture( tex_file, 0 )
	box:setScale( xcore.x3dpoint( 1, 1, 0 ) )
	box:setMaterialFlag( xcore.XMF_LIGHTING, false )

	-- particles = sim:addParticleSystem(10, "modules/tutorial_physics_2/resources/fire.bmp")

	tex_file = "modules/tutorial_physics_2/resources/particleblue.bmp"
	box2 = sim:addCubeMesh ( 1 )
	box2:addTexture( tex_file, 0 )
	box2:setScale( xcore.x3dpoint( 1, 1, 0 ) )
	box2:setMaterialFlag( xcore.XMF_LIGHTING, false )

	-- add earth
	mesh_file = "modules/tutorial_physics_2/resources/earth.x"
	tex_file = "modules/tutorial_physics_2/resources/earthmap1k.jpg"
	mesh = sim:addMesh ( 2, mesh_file )
	mesh:addTexture( tex_file, 0 )
	mesh:setScale( xcore.x3dpoint( 200, 200, 200 ) )
	mesh:setMaterialFlag( xcore.XMF_LIGHTING, false )

	np = xcore.x3dpoint( 300, -20, 0 )
	mesh:setPosition( np )
	-- TODO : add sphere geometry
	planet_body = xcore.xbody() -- sim:addBody ( 0, mesh_file, tex_file )??
	dyn:createBody( planet_body )
	dyn:createSphereGeom( planet_body, 200, np, xcore.x3dpoint( 0, 0, 0 ) ) -- size, pos, rot

	-- add a moon
	mesh_file = "modules/tutorial_physics_2/resources/earth.x"
	tex_file = "modules/tutorial_physics_2/resources/moonmap1k.jpg"
	moon = sim:addMesh ( 2, mesh_file )
	moon:addTexture( tex_file, 0 )
	moon:setScale( xcore.x3dpoint( 50, 50, 50 ) )
	moon:setMaterialFlag( xcore.XMF_LIGHTING, false )
	np = xcore.x3dpoint( -150, -20, 0 )
	moon:setPosition( np )
	-- TODO : add sphere geometry
	moon_body = xcore.xbody() -- sim:addBody ( 0, mesh_file, tex_file )??
	dyn:createBody( moon_body )
	dyn:createSphereGeom( moon_body, 50, np, xcore.x3dpoint( 0, 0, 0 ) ) -- size, pos, rot

	-- create geometry for player ship
	body = xcore.xbody() -- sim:addBody ( 0, mesh_file, tex_file )??
	dyn:createBody( body )
	dyn:createBoxGeom( body, xcore.x3dpoint( 10, 10, 10 ), xcore.x3dpoint( 0, 0, 0 ), xcore.x3dpoint( 0, 0, 90 ) ) -- size, pos, rot
	body:setMU( 100 )

	-- create geometry for pickup
	body2 = xcore.xbody() -- sim:addBody ( 0, mesh_file, tex_file )??
	dyn:createBody( body2 )
	dyn:createBoxGeom( body2, xcore.x3dpoint( 10, 10, 10 ), xcore.x3dpoint( 20, 0, 0 ), xcore.x3dpoint( 0, 0, 0 ) ) -- size, pos, rot
	body2:setMU( 0 )

	-- add a joint to restrict to 2d
	dyn:setPlane2d( body );
	dyn:setPlane2d( body2 );

	sim:bindkey ( "quit",  xcore.XKEY_ESCAPE  ) -- exit demo
	sim:bindkey ( "left",  xcore.XKEY_LEFT  )
	sim:bindkey ( "right", xcore.XKEY_RIGHT )
	sim:bindkey ( "up",    xcore.XKEY_UP    )
	sim:bindkey ( "down",  xcore.XKEY_DOWN  )
	sim:bindkey ( "zoom_in",  xcore.XKEY_PLUS  )
	sim:bindkey ( "zoom_out",  xcore.XKEY_MINUS  )
	sim:bindkey ( "shoot",  xcore.XKEY_SPACE  )

	-- add a container for lazer beams
	lazers = {}
	bodies = {}
	time = 0

	print "/end init"

end

function render()
	sim:render()
end

function update()
	sim:update()
	dyn:update()

	time = time + 1

	-- print ("pos:" .. body:getPosition().X .. ",".. body:getPosition().Y .. "," .. body:getPosition().Z )
	body:alignToAxis()
	body2:alignToAxis()

	body:damp(0.01)
	body2:damp(0.01)

	-- particles:setPosition( body:getPosition() )
	box:setPosition( body:getPosition() )
	box:setRotation( body:getRotation() )
	box2:setPosition( body2:getPosition() )
	box2:setRotation( body2:getRotation() )

	-- synch physics to graphics
	i = table.getn( lazers )
	for n = 1, i do
		lazers[n]:setPosition( bodies[n]:getPosition() ) -- should be updated automaticly with a link
		lazers[n]:setRotation( bodies[n]:getRotation() )
	end

	p = body:getPosition()

	cam:setTarget( p )
	cam:setPosition( xcore.x3dpoint( p.X, p.Y, p.Z-zoom ) )

	if sim:isKey( "quit" ) then sim:destroy() end

	if sim:isKey( "left" ) then
		-- print "turn left?"
		-- body:addForce( xcore.x3dpoint( -20000, 0, 0 ) )
		body:addRelForceAtRelPos( xcore.x3dpoint( 4000,0,0 ), xcore.x3dpoint( 0, -10, 0 ) );
	end

	if sim:isKey( "right" ) then
		-- body:addForce( xcore.x3dpoint( 20000, 0, 0 ) )
		body:addRelForceAtRelPos( xcore.x3dpoint( -4000,0,0 ), xcore.x3dpoint( 0, -10, 0 ) );

	end

	if sim:isKey( "up" ) then
		-- body:addForce( xcore.x3dpoint( 0, 20000, 0 ) )
		body:addRelForce( xcore.x3dpoint( 0,80000,0 ) );
	end

	if sim:isKey( "down" ) then
		-- body:addForce( xcore.x3dpoint( 0, -20000, 0 ) )
		body:addRelForce( xcore.x3dpoint( 0,-40000,0 ) );
	end

	if sim:isKey( "zoom_in" ) and zoom > 20 then zoom = zoom - 10 end
	if sim:isKey( "zoom_out" ) then zoom = zoom + 10 end

	if sim:isKey( "shoot" ) then
		-- print "shoot layzors!"
		shootLazer()
		-- grid:addForce ( xcore.x2dpoint(p.X, p.Y), 40000.0, 60.0, 0.1 )
	end

	-- print "collision"
	c = dyn:getCollision()
	if c ~= nil then
		-- print "collision!"
		b1 = c:geto1()
		b2 = c:geto2()
		-- print "a"
		if b1:getID() == body2:getID() then --  b2:getBody() == body2:getBody() then
			-- print "pickup!"
		end

		if b2:getID() == body2:getID() then
			-- print "pickup!"
		end
		-- print "b"
	end
	-- print "/collision"
end

function shootLazer() -- ownerid, type, pos, rot
	-- print "shoot lazer"

	if time > 100 then
		time = 0;
		-- print "make lazer"

		-- spawn lazers in front of ship (not on top!)
		user_pos = body:getPosition()
		user_rot = body:getRotation()

		m = xcore.xmatrix4()
		endpoint = xcore.x3dpoint( 0, 20, 0 )
		m:setRotationDegrees( user_rot )
		m:transformVect( endpoint )

		startpoint = user_pos
		endpoint.X = endpoint.X + startpoint.X
		endpoint.Y = endpoint.Y + startpoint.Y
		endpoint.Z = endpoint.Z + startpoint.Z

		-- create the 3d graphic
		i = table.getn( lazers ) + 1
		tex_file = "modules/tutorial_physics_2/resources/beam2.tga"
		lazers[ i ] = sim:addCubeMesh ( 0 )
		lazers[ i ]:addTexture( tex_file, 0 )

		np = body:getPosition()
		nr = body:getRotation()
		lazers[ i ]:setPosition( np )
		lazers[ i ]:setRotation( nr )
		lazers[ i ]:setScale( xcore.x3dpoint( .2, 1, .2 ) )
		lazers[ i ]:setMaterialFlag( xcore.XMF_LIGHTING, false )

		-- create geometry for lazer
		bodies[i] = xcore.xbody() -- sim:addBody ( 0, mesh_file, tex_file )??
		dyn:createBody( bodies[i] )
		dyn:createBoxGeom( bodies[i], xcore.x3dpoint( 10, 10, 10 ), endpoint, nr ) -- size, pos, rot
		-- dyn:createSphereGeom( bodies[i], 1, endpoint, nr  ) -- size, pos, rot
		bodies[i]:setMU( 100 )

		-- align to 2d plane
		dyn:setPlane2d( bodies[i] )
		bodies[i]:addRelForce( xcore.x3dpoint( 0, 10000000, 0 ) )

		-- print "end"
	end
	
end

-- end of file