-- state_game object
-- lwi
-- ...

state_game = state:new()

print "[state_game]"

-- creation
function state_game:new (o)
	print "[state_game] new state_game"
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	self.sid = 0

	-- add some static text describing the demo
	text = [[
	Xenos Engine Tutorial Statemachine
	
	The Current state is 'game'. We now use the code
	that we learned in the basic 3d tutorial to make a
	3d model and a level. But now we will also allow the 
	user to move the model around with the arrow keys.
	
	Press esc to return to the menu state.
	]]
	text_game = sim:addText( 0, text, 50, 50, 600, 600)
	text_game:setVisible( false )
		
	-- movement keys
	sim:bindkey ( "move_left",  xcore.XKEY_LEFT  )
	sim:bindkey ( "move_right", xcore.XKEY_RIGHT )
	sim:bindkey ( "move_forward",    xcore.XKEY_UP    )
	sim:bindkey ( "move_back",  xcore.XKEY_DOWN  )

	-- print a graph to the console
	-- sim:printTree();

	return o
end

-- called on state_game enter
function state_game:enter ()
	print "[state_game] enter state_game"
	self.time = 0
	text_game:setVisible( true )

	-- add a static camera
	camera = sim:addCamera ( 0, xcore.CAMERA_STATIC,-25,-60,-25, 25,-50, 25)
	
	-- create an animated 3d model
	mesh = sim:addMesh ( 0, "modules/tutorial_graphics_1/resources/faerie.md2" )
	mesh:addTexture ( "modules/tutorial_graphics_1/resources/faerie.bmp", 0 )
	mesh:setScale ( xcore.x3dpoint(1, 1, 1) )
	mesh:setMaterialFlag( xcore.XMF_LIGHTING, false )
	mesh:setMD2Animation( 0 )
	mesh:setAnimationSpeed( 10 )
	mesh:setPosition( xcore.x3dpoint( 25,-48, 25  ) )
	mesh:setRotation( xcore.x3dpoint( 0,180, 0  ) )
	
	-- add a .zip format Archive
	sim:addArchive( "modules/tutorial_graphics_1/resources/map-20kdm2.pk3" )
	
	-- create a 3d level mesh
	level_mesh = sim:addMesh ( 0, "20kdm2.bsp" )
	level_mesh:setMaterialFlag( xcore.XMF_LIGHTING, false )
	level_mesh:setPosition( xcore.x3dpoint( -1300,-144,-1249  ) )
	
end

-- called on state_game exit
function state_game:exit ()
	print "[state_game] exit state_game"
	self.time = 0

	-- hide text
	text_game:setVisible( false )

	-- clean up	
	mesh:destroy()
	level_mesh:destroy()
	camera:destroy()
	
end

-- called to advance state_game
function state_game:input ()
	-- print ( "[state_game] input" )

	-- update events
	r = self:doevents()

	-- draw everything
	sim:update()

	return r
end

-- called when changing state_game
function state_game:transition ( i ) -- transition to state_game v
	print "[state_game] transition"
end

-- This is a custom function to take care of events.
function state_game:doevents()
	s = 0.1;

	-- get mesh's position
	p = mesh:getPosition();
	
	if sim:isKey( "move_left" ) then 
		-- p = mesh:getPosition()
		if p.Z > -180 then
			self:move(0, -s)
		end
	end
	if sim:isKey( "move_right" ) then
		-- p = mesh:getPosition()
		if p.Z < 180 then
			self:move(0, s)
		end
	end
	if sim:isKey( "move_forward" ) then
		-- p = mesh:getPosition()
		if p.X < 130 then
			self:move(s, 0)
		end
	end
	if sim:isKey( "move_back" ) then
		-- p = mesh:getPosition()
		if p.X > -130 then
			self:move(-s, 0)
		end
	end
	-- if sim:isKey( "fire1" ) then print "fire1" end
	if sim:isKey( "key_pressed_exit_state" ) then return ENUM_MENU else return ENUM_GAME end
	return ENUM_GAME 
end

-- move the ship
function state_game:move(x, z)
 	-- print(string.format("pi = %.4f", PI))
	-- print ( string.format("node position: x%f, y%f ", p.X, p.Y) )
	p = mesh:getPosition()
	np = xcore.x3dpoint( p.X+x, p.Y, p.Z+z )
	mesh:setPosition( np )
end
